@@ -420,13 +420,23 @@ OPCODE_NAMES = {
 
 # ===----------------------------------------------------------------------===
 
+class BaseScriptError(Exception):
+    "Base class for all Script-related encode/decode exceptions"
+
+class BaseScriptDecodeError(BaseScriptError):
+    "Error while parsing bitcoin script"
+class EmptyScriptError(BaseScriptDecodeError):
+    "Attempted to retrieve opcode from empty script"
+class MissingPushDataError(BaseScriptDecodeError):
+    "End-of-script encountered while parsing multi-byte push code"
+
 class ScriptOp(SerializableMixin, six.binary_type):
     def __new__(cls, opcode=None, data=None, *args, **kwargs):
         # If just data is specified, then we automatically fill-in the opcode
         # as required to push the data on the stack.
         if opcode is None and data:
             len_ = len(data)
-            if 0 < len_ < OP_PUSHDATA1:
+            if 0 <= len_ < OP_PUSHDATA1:
                 opcode = len_
             elif OP_PUSHDATA1 <= len_ <= 0xff:
                 opcode = OP_PUSHDATA1
@@ -435,10 +445,7 @@ class ScriptOp(SerializableMixin, six.binary_type):
             elif 0xffff < len_ < 0xffffffff:
                 opcode = OP_PUSHDATA4
             else:
-                if not len_:
-                    raise TypeError(u"cannot push empty string")
-                else:
-                    raise TypeError(u"string exceeds maximum length")
+                raise TypeError(u"pushed data exceeds maximum length")
 
         # The push opcode encodes the length of the data to be pushed, or
         # the size of the length which directly follows. In either case we
@@ -446,7 +453,7 @@ class ScriptOp(SerializableMixin, six.binary_type):
         # this limit.
         if opcode==OP_INVALIDOPCODE:
             raise ValueError(u"invalid opcode")
-        elif opcode>0 and opcode<OP_PUSHDATA1 and opcode!=len(data):
+        elif 0 < opcode < OP_PUSHDATA1 and opcode!=len(data):
             raise ValueError(u"opcode/data-length mismatch")
         elif (opcode==OP_PUSHDATA1 and len(data)>0xff or
               opcode==OP_PUSHDATA2 and len(data)>0xffff or
@@ -454,13 +461,13 @@ class ScriptOp(SerializableMixin, six.binary_type):
             raise ValueError(u"data length exceeds serialization limit")
 
         result = pack('<B', opcode)
-        if opcode==OP_PUSHDATA1:
+        if opcode == OP_PUSHDATA1:
             result += pack('<B', len(data))
-        elif opcode==OP_PUSHDATA2:
+        elif opcode == OP_PUSHDATA2:
             result += pack('<H', len(data))
-        elif opcode==OP_PUSHDATA4:
+        elif opcode == OP_PUSHDATA4:
             result += pack('<I', len(data))
-        if opcode>0 and opcode<=OP_PUSHDATA4:
+        if 0 < opcode <= OP_PUSHDATA4:
             result += data
 
         return super(ScriptOp, cls).__new__(cls, result, *args, **kwargs)
@@ -481,36 +488,37 @@ class ScriptOp(SerializableMixin, six.binary_type):
     @classmethod
     def deserialize(cls, file_):
         opcode = file_.read(1)
-        if not len(opcode): raise StopIteration
+        if not len(opcode):
+            raise EmptyScriptError
         opcode = unpack('<B', opcode)[0]
 
         datalen = 0
-        if opcode<OP_PUSHDATA1:
+        if opcode < OP_PUSHDATA1:
             datalen = opcode
-        elif opcode==OP_PUSHDATA1:
+        elif opcode == OP_PUSHDATA1:
             datalen = unpack('<B', file_.read(1))[0]
-        elif opcode==OP_PUSHDATA2:
+        elif opcode == OP_PUSHDATA2:
             datalen = unpack('<H', file_.read(2))[0]
-        elif opcode==OP_PUSHDATA4:
+        elif opcode == OP_PUSHDATA4:
             datalen = unpack('<I', file_.read(4))[0]
         if datalen:
             data = file_.read(datalen)
         else:
-            data = ''
+            data = b''
         if len(data) != datalen:
-            raise ValueError(u"unexpected end-of-file in data string")
+            raise MissingPushDataError
 
-        if opcode<=OP_PUSHDATA4:
-            return cls(opcode, data)
+        if 0 < opcode <= OP_PUSHDATA4:
+            return cls(opcode, data=data)
         return cls(opcode)
 
     @property
     def boolean(self):
-        if self.opcode==OP_0:
+        if self.opcode == OP_0:
             return False
-        if self.opcode==OP_1NEGATE:
+        if self.opcode == OP_1NEGATE:
             return True
-        if self.opcode>=OP_1 and self.opcode<=OP_16:
+        if OP_1 <= self.opcode <= OP_16:
             return True
         if self.opcode in xrange(1,OP_PUSHDATA4+1):
             return (
@@ -528,11 +536,11 @@ class ScriptOp(SerializableMixin, six.binary_type):
 
     @property
     def integral(self):
-        if self.opcode==OP_1NEGATE:
+        if self.opcode == OP_1NEGATE:
             return -1
-        elif self.opcode==OP_0:
+        elif self.opcode == OP_0:
             return 0
-        elif self.opcode>=OP_1 and self.opcode<=OP_16:
+        elif OP_1 <= self.opcode <= OP_16:
             return self.opcode - OP_1 + 1
         elif self.opcode in xrange(1,OP_PUSHDATA4+1):
             data = self.data[:-1] + chr(ord(self.data[-1])&0x7f)
@@ -550,7 +558,7 @@ class ScriptOp(SerializableMixin, six.binary_type):
             self.opcode = OP_1NEGATE
         elif value == 0:
             self.opcode = OP_0
-        elif value >= 1 and value <= 16:
+        elif 1 <= value <= 16:
             self.opcode = OP_1 + value - 1
         else:
             neg = value < 1
@@ -571,40 +579,43 @@ class ScriptOp(SerializableMixin, six.binary_type):
             self.data = data
 
     def __repr__(self):
+        data_str = ''.join([
+            '\'',
+            (self.data or '').encode('hex'),
+            '\'.decode(\'hex\')'])
         if self.opcode>0 and self.opcode<=OP_PUSHDATA4:
-            return ''.join(['\'', self.data.encode('hex'), '\'.decode(\'hex\')'])
+            return data_str
         else:
-            if self.opcode in OPCODE_NAMES:
+            if self.opcode in OPCODE_NAMES and self.data is None:
                 return OPCODE_NAMES[self.opcode]
             else:
-                return "OP_UNKNOWN"
+                if self.data is None:
+                    data_str = ''
+                else:
+                    data_str = ', data=' + data_str
+                return 'ScriptOp(%d%s)' % (self.opcode, data_str)
 
 # ===----------------------------------------------------------------------===
 
-class Script(SerializableMixin, tuple):
-    def __new__(cls, *args, **kwargs):
-        if len(args)==1 and isinstance(args[0], str) and not kwargs:
-            return cls.deserialize(StringIO(args[0]))
-        return super(Script, cls).__new__(cls, *args, **kwargs)
+class Script(SerializableMixin, six.binary_type):
+    def __iter__(self):
+        script_op_class = getattr(self, 'get_script_op_class', lambda:
+                          getattr(self, 'script_op_class', ScriptOp))()
+        file_ = StringIO(self)
+        while True:
+            yield script_op_class.deserialize(file_)
 
     def serialize(self):
-        return serialize_varchar(''.join(map(lambda op:op.serialize(), self)))
+        return serialize_varchar(self)
     @classmethod
     def deserialize(cls, file_):
-        l = []
-        input_ = StringIO(deserialize_varchar(file_))
-        try:
-            while True:
-                l.append(ScriptOp.deserialize(input_))
-        except StopIteration: pass
-        return cls(l)
+        return cls(deserialize_varchar(file_))
 
-    def __eq__(self, other, *args, **kwargs):
-        if isinstance(other, str):
-            return self.serialize() == other
-        return super(Script, self).__eq__(other, *args, **kwargs)
     def __repr__(self):
-        return u"Script([%s])" % ', '.join(map(repr, self))
+        try:
+            return u"Script([%s])" % ', '.join(map(repr, self))
+        except:
+            return u"Script(\'%s\'.decode(\'hex\'))" % self.encode('hex')
 
 # ===----------------------------------------------------------------------===
 
